@@ -65,6 +65,25 @@ class BitcoinException(Exception):
         self.code = error['code']
 
 
+class TransportException(Exception):
+    """
+    Class to define transport-level failures.
+    """
+    def __init__(self, msg, code=None, protocol=None, raw_detail=None):
+        self.msg = msg
+        self.code = code
+        self.protocol = protocol
+        self.raw_detail = raw_detail
+        self.s = """
+        Transport-level failure: {msg}
+        Code: {code}
+        Protocol: {protocol}
+        """.format(msg=msg, code=code, protocol=protocol)
+
+    def __str__(self):
+        return self.s
+
+
 ##### General application defined errors
 class SafeMode(BitcoinException):
     """
@@ -126,11 +145,13 @@ class WalletError(BitcoinException):
     """
 SendError = WalletError  # Backwards compatibility
 
+
 class InsufficientFunds(WalletError):
     """
     Insufficient funds to complete transaction in wallet or account
     """
-    
+
+
 class InvalidAccountName(WalletError):
     """
     Invalid account name
@@ -172,7 +193,7 @@ class WalletAlreadyUnlocked(WalletError):
     Wallet is already unlocked
     """
 
-    
+
 # For convenience, we define more specific exception classes
 # for the more common errors.
 _exception_map = {
@@ -196,8 +217,11 @@ _exception_map = {
 }
 
 
-def _wrap_exception(error):
+def wrap_exception(error):
     """
     Convert a JSON error object to a more specific Bitcoin exception.
     """
+    # work around to temporarily fix https://github.com/bitcoin/bitcoin/issues/3007
+    if error['code'] == BitcoinException.WALLET_ERROR and error['message'] == 'Insufficient funds':
+        error['code'] = BitcoinException.WALLET_INSUFFICIENT_FUNDS
     return _exception_map.get(error['code'], BitcoinException)(error)
